Package: karyoploteR
Type: Package
Title: Plot customizable linear genomes displaying arbitrary data
Version: 1.37.0
Date: 2025-06-20
Authors@R: 
  c(person(given = "Bernat",
           family = "Gel",
           role = c("aut", "cre"),
           email = "bgel@igtp.cat",
           comment = c(ORCID = "0000-0001-8878-349X")))
Description: karyoploteR creates karyotype plots of arbitrary genomes and offers
    a complete set of functions to plot arbitrary data on them. It mimicks many
    R base graphics functions coupling them with a coordinate change function 
    automatically mapping the chromosome and data coordinates into the plot 
    coordinates. In addition to the provided data plotting functions, it is 
    easy to add new ones.
License: Artistic-2.0
Depends: R (>= 3.4), regioneR, GenomicRanges, methods
Imports: regioneR, GenomicRanges, IRanges, Rsamtools, stats, graphics,
        memoise, rtracklayer, Seqinfo, GenomeInfoDb, S4Vectors,
        biovizBase, digest, bezier, GenomicFeatures, bamsignals,
        AnnotationDbi, grDevices, VariantAnnotation
Suggests: BiocStyle, knitr, rmarkdown, markdown, testthat, magrittr,
        BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Hsapiens.UCSC.hg19.masked,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene, org.Hs.eg.db, org.Mm.eg.db,
        pasillaBamSubset
URL: https://github.com/bernatgel/karyoploteR
BugReports: https://github.com/bernatgel/karyoploteR/issues
VignetteBuilder: knitr
biocViews: Visualization, CopyNumberVariation, Sequencing, Coverage,
        DNASeq, ChIPSeq, MethylSeq, DataImport, OneChannel
NeedsCompilation: no
Encoding: UTF-8
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/karyoploteR
git_branch: devel
git_last_commit: fd32b1e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
Packaged: 2025-11-04 23:14:26 UTC; biocbuild
Author: Bernat Gel [aut, cre] (ORCID: <https://orcid.org/0000-0001-8878-349X>)
Maintainer: Bernat Gel <bgel@igtp.cat>
